
#include "comb/permtrotter.h"

#include "perm/perminvert.h"

#include "fxttypes.h"
#include "fxtio.h"
#include "demo/nextarg.h"


//% Generate all permutations in minimal-change order using Trotter's algorithm.

//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 4;
    NXARG(n, "Permutations of n elements (n>=2).");
    if ( n<2 )  n = 2;
    bool dfz = true;  // whether to print dots for zeros

    perm_trotter P(n);
#ifdef TIMING
    while ( P.next() )  {;}
#else
    const ulong *x = P.data();
    ulong *xi = new ulong[n];
    ulong ct = 0;
    do
    {
        cout << setw(4) << ct << ":";
        ++ct;

        P.print("    ", dfz);

        ulong sw1, sw2;
        P.get_swap(sw1, sw2);
        cout << "    (" << sw1 << ", " << sw2 << ") ";

        make_inverse(x, xi, n);
        print_perm("    ", xi, n, dfz);

        cout << endl;
    }
    while ( P.next() );
#endif

    return 0;
}
// -------------------------

/*
 Timing:
 time ./bin 12
./bin 12  3.39s user 0.01s system 100% cpu 3.400 total
 12!/3.400 == 140,882,823 permutations per second
 ==> about 15 cycles per update
*/
